@@ -47,6 +47,10 @@ void test_remove_unix_socket_on_close (void);
 void test_remove_unix_socket_on_create_accessor (void);
 void test_remove_unix_socket_on_create (void);
 void test_not_remove_unix_socket_on_create (void);
+void test_remove_pid_file_on_exit_accessor (void);
+void test_remove_pid_file_on_exit (void);
+void test_syslog_identify_accessor (void);
+void test_syslog_facility_accessor (void);
 void test_suspend_time_on_unacceptable (void);
 void test_max_connections (void);
 void test_effective_user (void);
@@ -334,7 +338,7 @@ cut_setup (void)
         milter_client_set_n_workers(client, n_workers);
     }
     loop_run_count = 0;
-    loop = milter_client_get_process_loop(client);
+    loop = milter_client_get_event_loop(client);
     g_object_ref(loop);
     milter_event_loop_set_custom_run_func(loop, loop_run);
     setup_client_signals();
@@ -821,6 +825,66 @@ test_not_remove_unix_socket_on_create (void)
     gcut_assert_equal_error(expected_error, actual_error);
 }
 
+void
+test_remove_pid_file_on_exit_accessor (void)
+{
+    cut_assert_true(milter_client_is_remove_pid_file_on_exit(client));
+    milter_client_set_remove_pid_file_on_exit(client, FALSE);
+    cut_assert_false(milter_client_is_remove_pid_file_on_exit(client));
+}
+
+static const gchar *
+pid_file_path (void)
+{
+    return cut_take_printf("%s/milter.pid", tmp_dir);
+}
+
+static gboolean
+cb_idle_check_pid_file_existence (gpointer user_data)
+{
+    gboolean *pid_file_exist = user_data;
+
+    *pid_file_exist = g_file_test(pid_file_path(), G_FILE_TEST_EXISTS);
+
+    return FALSE;
+}
+
+void
+test_remove_pid_file_on_exit (void)
+{
+    gboolean pid_file_exist = FALSE;
+
+    cut_trace(setup_client());
+    milter_client_set_pid_file(client, pid_file_path());
+    milter_event_loop_add_idle_full(loop,
+                                    G_PRIORITY_HIGH,
+                                    cb_idle_check_pid_file_existence,
+                                    &pid_file_exist,
+                                    NULL);
+    cut_assert_true(milter_client_run(client, &actual_error));
+    gcut_assert_error(actual_error);
+    cut_assert_true(pid_file_exist);
+
+    cut_assert_false(g_file_test(pid_file_path(), G_FILE_TEST_EXISTS));
+}
+
+void
+test_syslog_identify_accessor (void)
+{
+    cut_assert_equal_string(NULL, milter_client_get_syslog_identify(client));
+    milter_client_set_syslog_identify(client, "milter-test-client");
+    cut_assert_equal_string("milter-test-client",
+                            milter_client_get_syslog_identify(client));
+}
+
+void
+test_syslog_facility_accessor (void)
+{
+    cut_assert_equal_string(NULL, milter_client_get_syslog_facility(client));
+    milter_client_set_syslog_facility(client, "mail");
+    cut_assert_equal_string("mail", milter_client_get_syslog_facility(client));
+}
+
 void
 test_suspend_time_on_unacceptable (void)
 {
@@ -878,7 +942,7 @@ test_event_loop_backend (void)
 {
     gcut_assert_equal_enum(
         MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
-        MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB,
+        MILTER_CLIENT_EVENT_LOOP_BACKEND_DEFAULT,
         milter_client_get_event_loop_backend(client));
     milter_client_set_event_loop_backend(client,
                                          MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV);
