@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2010-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,10 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#ifdef HAVE_LOCALE_H
+#  include <locale.h>
+#endif
+
 #include <libmilter/mfapi.h>
 
 static gchar *spec = NULL;
@@ -56,13 +60,12 @@ static const GOptionEntry option_entries[] =
     {"connection-spec", 's', 0, G_OPTION_ARG_STRING, &spec,
      N_("The spec of socket. (unix:PATH|inet:PORT[@HOST]|inet6:PORT[@HOST])"),
      "SPEC"},
-    {"verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &verbose,
+    {"verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
      N_("Be verbose"), NULL},
-    {"no-report-request", 0, G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_REVERSE,
-     G_OPTION_ARG_NONE, &report_request,
+    {"no-report-request", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,
+     &report_request,
      N_("Don't report request values"), NULL},
-    {"report-memory-profile", 0, G_OPTION_FLAG_NO_ARG,
-     G_OPTION_ARG_NONE, &report_memory_profile,
+    {"report-memory-profile", 0, 0, G_OPTION_ARG_NONE, &report_memory_profile,
      N_("Report memory profile. "
         "Need to set MILTER_MEMORY_PROFILE=yes environment variable."), NULL},
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
@@ -354,6 +357,10 @@ main (int argc, char *argv[])
     GOptionGroup *main_group;
     unsigned int major, minor, patch_level;
 
+#ifdef HAVE_LOCALE_H
+    setlocale(LC_ALL, "");
+#endif
+
     /*
      * workaround for memory profiler for GLib memory
      * profiler, we need to call g_mem_set_vtable prior to
